#!/bin/bash
#

# build_mpich.sh <product_directory> <e2|e4> <debug|opt|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e2|e4> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=mpich
origpkgver=v3_0_4
pkgver=${origpkgver}a
ssibuildshims_version=v0_07_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=mpich-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS}

# Common configure args

# Note: building CXX: caveat emptor.
common_config_args="--prefix=${pkgdir}
 --enable-f77 --enable-fc --enable-cxx --enable-romio --enable-debuginfo
 --enable-smpcoll --enable-versioning --enable-threads=runtime
 --disable-wrapper-rpath --enable-shared --enable-static --disable-libxml2
 --enable-timer-type=clock_gettime --enable-g=debug"

if [[ "${OS1}" == Darwin ]]; then
  common_config_args="${common_config_args}
 --enable-timer-type=mach_absolute_time"
fi

# Debug configure args
debug_config_args="--enable-error-checking=all --enable-error-messages=all
 --enable-timing=all --enable-mpit-pvars=all
 --disable-fast"

# Profile configure args
prof_config_args=" --enable-mpit-pvars=none --enable-strict
 --enable-fast=nochkmsg,notiming,ndebug,nompit,O3"

if [ "${extraqual}" = "debug" ]
then
    extra_config_args="${debug_config_args}"
elif [ "${extraqual}" = "prof" ]
then
    extra_config_args="${prof_config_args}"
fi


mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x 

cd "${sourcedir}"
tar xf ${tardir}/${pkgtarname}.tar.gz

cd ${blddir}
# Configure
${sourcedir}/${pkgtarname}/configure ${common_config_args} ${extra_config_args} || die "ERROR: configure failed."

make -j ${ncore} all || die "ERROR: make failed."

# Need to re-link execs to avoid dynamic dependency on mpl.
#
# Yes, this is absolutely horrible and prone to change with the wind and
# the price of rice. libtool is truly evil.
cd src/pm/hydra
for i in hydra_{nameserver,persist,pmi_proxy} mpiexec.hydra; do
  out=.libs/${i}
  sources=${i%%.*}*.o
  libs=".libs/libhydra.a ../../mpl/.libs/libmpl.a tools/topo/hwloc/hwloc/src/.libs/libhwloc_embedded.a -lpthread -lm"
  case ${out} in
      */mpiexec.hydra)
          sources="${sources} utils.o"
          libs=".libs/libpm.a ${libs}"
          ;;
      *)
          :
  esac
  set -x
  gcc -o ${out} ${sources} ${libs} || die "ERROR: unable to re-link execs."
  set +x
done
cd - 2>&1 >/dev/null

make install || die "ERROR: make install failed."

set +x

# Clean up.
cd "${pkgdir}"

# Wrapper scripts and config files.
find bin etc -type f | while read file; do
  [[ "$(file "${file}")" == *"text"* ]] && \
   perl -wapi\~ -e 's&\Q'"${pkgdir}"'\E&\${MPICH_FQ_DIR}&g' "${file}"
done

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "mpich is installed at ${MPICH_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
